<?php
/**
 * License Module
 *
 * @package blesta
 * @subpackage blesta.components.modules.license_module
 * @copyright Copyright (c) 2010, Phillips Data, Inc.
 * @license http://www.blesta.com/license/ The Blesta License Agreement
 * @link http://www.blesta.com/ Blesta
 */
class LicenseModule extends Module {
	
	/**
	 * @var string The version of this module
	 */
	private static $version = "1.1.1";
	/**
	 * @var string The authors of this module
	 */
	private static $authors = array(array('name'=>"Phillips Data, Inc.",'url'=>"http://www.blesta.com"));
	
	/**
	 * Initializes the module
	 */
	public function __construct() {
		// Load components required by this module
		Loader::loadComponents($this, array("Input"));
		
		// Load the language required by this module
		Language::loadLang("license_module", null, dirname(__FILE__) . DS . "language" . DS);
	}
	
	/**
	 * Returns the name of this module
	 *
	 * @return string The common name of this module
	 */
	public function getName() {
		return Language::_("LicenseModule.name", true);
	}
	
	/**
	 * Returns the version of this module
	 *
	 * @return string The current version of this module
	 */
	public function getVersion() {
		return self::$version;
	}
	
	/**
	 * Returns the name and url of the authors of this module
	 *
	 * @return array The name and url of the authors of this module
	 */
	public function getAuthors() {
		return self::$authors;
	}
	
	/**
	 * Returns the value used to identify a particular service
	 *
	 * @param stdClass $service A stdClass object representing the service
	 * @return string A value used to identify this service amongst other similar services
	 */
	public function getServiceName($service) {
		foreach ($service->fields as $field) {
			if ($field->key == "license_module_key")
				return $field->value;
		}
		return null;
	}
	
	/**
	 * Returns a noun used to refer to a module row
	 *
	 * @return string The noun used to refer to a module row
	 */
	public function moduleRowName() {
		return Language::_("LicenseModule.module_row", true);
	}
	
	/**
	 * Returns a noun used to refer to a module row in plural form
	 *
	 * @return string The noun used to refer to a module row in plural form
	 */
	public function moduleRowNamePlural() {
		return Language::_("LicenseModule.module_row_plural", true);
	}
	
	/**
	 * Returns a noun used to refer to a module group
	 *
	 * @return string The noun used to refer to a module group
	 */
	public function moduleGroupName() {
		return Language::_("LicenseModule.module_group", true);
	}
	
	/**
	 * Returns the key used to identify the primary field from the set of module row meta fields.
	 *
	 * @return string The key used to identify the primary field from the set of module row meta fields
	 */
	public function moduleRowMetaKey() {
		return "name";
	}

	/**
	 * Adds the service to the remote server. Sets Input errors on failure,
	 * preventing the service from being added.
	 *
	 * @param stdClass $package A stdClass object representing the selected package
	 * @param array $vars An array of user supplied info to satisfy the request
	 * @param stdClass $parent_package A stdClass object representing the parent service's selected package (if the current service is an addon service)
	 * @param stdClass $parent_service A stdClass object representing the parent service of the service being added (if the current service is an addon service service and parent service has already been provisioned)
	 * @param string $status The status of the service being added. These include:
	 * 	- active
	 * 	- canceled
	 * 	- pending
	 * 	- suspended
	 * @return array A numerically indexed array of meta fields to be stored for this service containing:
	 * 	- key The key for this meta field
	 * 	- value The value for this key
	 * 	- encrypted Whether or not this field should be encrypted (default 0, not encrypted)
	 * @see Module::getModule()
	 * @see Module::getModuleRow()
	 */
	public function addService($package, array $vars=null, $parent_package=null, $parent_service=null, $status="pending") {
		$row = $this->getModuleRow();
		$license_key = null;
		$pool = $row->meta->key_pool;
		
		if ($status == "active") {
			if ($row->meta->key_method == "auto") {
				$exists = true;
				for ($i = 0; $i < 100 && $exists; $i++) {
					
					$key = null;
					for ($j = 0; $j < $row->meta->key_length; $j++)
						$key .= $pool[mt_rand(0, strlen($pool)-1)];
					
					$license_key = str_replace("{key}", $key, $row->meta->key_format);
					
					$exists = $this->getLicense($row->module_id, $license_key);
				}
				if ($exists) {
					$this->Input->setErrors(array(
						'license_module_key' => Language::_('LicenseModule.!error.license_module_key.not_unique', true)
					));
					return;
				}
			}
			else {
				#
				# TODO: Handle a list of license keys
				#
			}
		}
		
		return array(
			array(
				'key' => "license_module_key",
				'value' => $license_key,
			),
			array(
				'key' => "license_module_domains",
				'value' => array(),
			),
			array(
				'key' => "license_module_ips",
				'value' => array(),
			),
			array(
				'key' => "license_module_paths",
				'value' => array(),
			),
			array(
				'key' => "license_module_status",
				'value' => "reissue",
			),
			array(
				'key' => "license_module_callhome",
				'value' => "",
			),
			array(
				'key' => "license_module_version",
				'value' => "",
			)
		);
	}
	
	/**
	 * Edits the service on the remote server. Sets Input errors on failure,
	 * preventing the service from being edited.
	 *
	 * @param stdClass $package A stdClass object representing the current package
	 * @param stdClass $service A stdClass object representing the current service
	 * @param array $vars An array of user supplied info to satisfy the request
	 * @param stdClass $parent_package A stdClass object representing the parent service's selected package (if the current service is an addon service)
	 * @param stdClass $parent_service A stdClass object representing the parent service of the service being edited (if the current service is an addon service)
	 * @return array A numerically indexed array of meta fields to be stored for this service containing:
	 * 	- key The key for this meta field
	 * 	- value The value for this key
	 * 	- encrypted Whether or not this field should be encrypted (default 0, not encrypted)
	 * @see Module::getModule()
	 * @see Module::getModuleRow()
	 */
	public function editService($package, $service, array $vars=array(), $parent_package=null, $parent_service=null) {
		
		#
		# TODO: Update license key, ensure no other service has it
		#
		
		$service_fields = $this->serviceFieldsToObject($service->fields);
		
		if (!isset($vars['license_module_status']))
			$vars['license_module_status'] = "";
		
		// Update fields
		$service_fields->license_module_status = $vars['license_module_status'];
		
		return array(
			array(
				'key' => "license_module_key",
				'value' => $service_fields->license_module_key,
			),
			array(
				'key' => "license_module_domains",
				'value' => $service_fields->license_module_domains,
			),
			array(
				'key' => "license_module_ips",
				'value' => $service_fields->license_module_ips,
			),
			array(
				'key' => "license_module_paths",
				'value' => $service_fields->license_module_paths,
			),
			array(
				'key' => "license_module_status",
				'value' => $service_fields->license_module_status,
			),
			array(
				'key' => "license_module_callhome",
				'value' => $service_fields->license_module_callhome,
			),
			array(
				'key' => "license_module_version",
				'value' => isset($service_fields->license_module_version) ? $service_fields->license_module_version : "",
			)
		);
	}
	
	/**
	 * Cancels the service on the remote server. Sets Input errors on failure,
	 * preventing the service from being canceled.
	 *
	 * @param stdClass $package A stdClass object representing the current package
	 * @param stdClass $service A stdClass object representing the current service
	 * @param stdClass $parent_package A stdClass object representing the parent service's selected package (if the current service is an addon service)
	 * @param stdClass $parent_service A stdClass object representing the parent service of the service being canceled (if the current service is an addon service)
	 * @return mixed null to maintain the existing meta fields or a numerically indexed array of meta fields to be stored for this service containing:
	 * 	- key The key for this meta field
	 * 	- value The value for this key
	 * 	- encrypted Whether or not this field should be encrypted (default 0, not encrypted)
	 * @see Module::getModule()
	 * @see Module::getModuleRow()
	 */
	public function cancelService($package, $service, $parent_package=null, $parent_service=null) {
		// Nothing to do
		return null;
	}
	
	/**
	 * Suspends the service on the remote server. Sets Input errors on failure,
	 * preventing the service from being suspended.
	 *
	 * @param stdClass $package A stdClass object representing the current package
	 * @param stdClass $service A stdClass object representing the current service
	 * @param stdClass $parent_package A stdClass object representing the parent service's selected package (if the current service is an addon service)
	 * @param stdClass $parent_service A stdClass object representing the parent service of the service being suspended (if the current service is an addon service)
	 * @return mixed null to maintain the existing meta fields or a numerically indexed array of meta fields to be stored for this service containing:
	 * 	- key The key for this meta field
	 * 	- value The value for this key
	 * 	- encrypted Whether or not this field should be encrypted (default 0, not encrypted)
	 * @see Module::getModule()
	 * @see Module::getModuleRow()
	 */
	public function suspendService($package, $service, $parent_package=null, $parent_service=null) {
		// Nothing to do
		return null;
	}
	
	/**
	 * Unsuspends the service on the remote server. Sets Input errors on failure,
	 * preventing the service from being unsuspended.
	 *
	 * @param stdClass $package A stdClass object representing the current package
	 * @param stdClass $service A stdClass object representing the current service
	 * @param stdClass $parent_package A stdClass object representing the parent service's selected package (if the current service is an addon service)
	 * @param stdClass $parent_service A stdClass object representing the parent service of the service being unsuspended (if the current service is an addon service)
	 * @return mixed null to maintain the existing meta fields or a numerically indexed array of meta fields to be stored for this service containing:
	 * 	- key The key for this meta field
	 * 	- value The value for this key
	 * 	- encrypted Whether or not this field should be encrypted (default 0, not encrypted)
	 * @see Module::getModule()
	 * @see Module::getModuleRow()
	 */
	public function unsuspendService($package, $service, $parent_package=null, $parent_service=null) {
		// Nothing to do
		return null;
	}
	
	/**
	 * Returns the rendered view of the manage module page
	 *
	 * @param mixed $module A stdClass object representing the module and its rows
	 * @param array $vars An array of post data submitted to or on the manager module page (used to repopulate fields after an error)
	 * @return string HTML content containing information to display when viewing the manager module page
	 */
	public function manageModule($module, array &$vars) {
		// Load the view into this object, so helpers can be automatically added to the view
		$this->view = new View("manage", "default");
		$this->view->base_uri = $this->base_uri;
		$this->view->setDefaultView("components" . DS . "modules" . DS . "license_module" . DS);
		
		// Load the helpers required for this view
		Loader::loadHelpers($this, array("Form", "Html", "Widget"));

		$this->view->set("module", $module);
		
		return $this->view->fetch();
	}
	
	/**
	 * Returns the rendered view of the add module row page
	 *
	 * @param array $vars An array of post data submitted to or on the add module row page (used to repopulate fields after an error)
	 * @return string HTML content containing information to display when viewing the add module row page
	 */
	public function manageAddRow(array &$vars) {
		Loader::loadModels($this, array("Packages"));
		
		// Load the view into this object, so helpers can be automatically added to the view
		$this->view = new View("add_row", "default");
		$this->view->base_uri = $this->base_uri;
		$this->view->setDefaultView("components" . DS . "modules" . DS . "license_module" . DS);
		
		// Load the helpers required for this view
		Loader::loadHelpers($this, array("Form", "Html", "Widget"));
		
		$addon_packages = $this->Form->collapseObjectArray($this->Packages->getAll(Configure::get("Blesta.company_id"), array('name'=>"ASC"), null, "addon"), "name", "id");
		
		$this->view->set("vars", (object)$vars);
		$this->view->set("addon_packages", array('' => Language::_("LicenseModule.select_none", true)) + $addon_packages);
		$this->view->set("license_methods", $this->licenseMethods());
		$this->view->set("license_key_methods", $this->licenseKeyMethods());
		return $this->view->fetch();
	}

	/**
	 * Returns the rendered view of the edit module row page
	 *
	 * @param stdClass $module_row The stdClass representation of the existing module row
	 * @param array $vars An array of post data submitted to or on the edit module row page (used to repopulate fields after an error)
	 * @return string HTML content containing information to display when viewing the edit module row page
	 */	
	public function manageEditRow($module_row, array &$vars) {
		Loader::loadModels($this, array("Packages"));
		
		// Load the view into this object, so helpers can be automatically added to the view
		$this->view = new View("edit_row", "default");
		$this->view->base_uri = $this->base_uri;
		$this->view->setDefaultView("components" . DS . "modules" . DS . "license_module" . DS);
		
		// Load the helpers required for this view
		Loader::loadHelpers($this, array("Form", "Html", "Widget"));
		
		if (empty($vars))
			$vars = $module_row->meta;
		
		$addon_packages = $this->Form->collapseObjectArray($this->Packages->getAll(Configure::get("Blesta.company_id"), array('name'=>"ASC"), null, "addon"), "name", "id");
		
		$this->view->set("vars", (object)$vars);
		$this->view->set("addon_packages", array('' => Language::_("LicenseModule.select_none", true)) + $addon_packages);
		$this->view->set("license_methods", $this->licenseMethods());
		$this->view->set("license_key_methods", $this->licenseKeyMethods());
		return $this->view->fetch();
	}
	
	/**
	 * Adds the module row on the remote server. Sets Input errors on failure,
	 * preventing the row from being added.
	 *
	 * @param array $vars An array of module info to add
	 * @return array A numerically indexed array of meta fields for the module row containing:
	 * 	- key The key for this meta field
	 * 	- value The value for this key
	 * 	- encrypted Whether or not this field should be encrypted (default 0, not encrypted)
	 */
	public function addModuleRow(array &$vars) {
		
		#
		# TODO: Do error checking
		#
		
		$meta = array();
		foreach ($vars as $key => $value) {
			$meta[] = array(
				'key' => $key,
				'value' => $value,
				'encrypted' => 0
			);
		}
		return $meta;
	}
	
	/**
	 * Edits the module row on the remote server. Sets Input errors on failure,
	 * preventing the row from being updated.
	 *
	 * @param stdClass $module_row The stdClass representation of the existing module row
	 * @param array $vars An array of module info to update
	 * @return array A numerically indexed array of meta fields for the module row containing:
	 * 	- key The key for this meta field
	 * 	- value The value for this key
	 * 	- encrypted Whether or not this field should be encrypted (default 0, not encrypted)
	 */
	public function editModuleRow($module_row, array &$vars) {
		
		#
		# TODO: Do error checking
		#
		
		$meta = array();
		foreach ($vars as $key => $value) {
			$meta[] = array(
				'key'=>$key,
				'value'=>$value,
				'encrypted'=>0
			);
		}
		return $meta;
	}
	
	/**
	 * Deletes the module row on the remote server. Sets Input errors on failure,
	 * preventing the row from being deleted.
	 *
	 * @param stdClass $module_row The stdClass representation of the existing module row
	 */
	public function deleteModuleRow($module_row) {
		// Nothing to do
		
	}
	
	/**
	 * Returns all fields used when adding/editing a package, including any
	 * javascript to execute when the page is rendered with these fields.
	 *
	 * @param $vars stdClass A stdClass object representing a set of post fields
	 * @return ModuleFields A ModuleFields object, containg the fields to render as well as any additional HTML markup to include
	 */
	public function getPackageFields($vars=null) {
		return new ModuleFields();
	}
	
	/**
	 * Returns an array of key values for fields stored for a module, package,
	 * and service under this module, used to substitute those keys with their
	 * actual module, package, or service meta values in related emails.
	 *
	 * @return array A multi-dimensional array of key/value pairs where each key is one of 'module', 'package', or 'service' and each value is a numerically indexed array of key values that match meta fields under that category.
	 * @see Modules::addModuleRow()
	 * @see Modules::editModuleRow()
	 * @see Modules::addPackage()
	 * @see Modules::editPackage()
	 * @see Modules::addService()
	 * @see Modules::editService()
	 */
	public function getEmailTags() {
		return array(
			'module' => array('name', 'label'),
			'package' => array('name'),
			'service' => array('license_module_key')
		);
	}

	/**
	 * Returns all fields to display to an admin attempting to add a service with the module
	 *
	 * @param stdClass $package A stdClass object representing the selected package
	 * @param $vars stdClass A stdClass object representing a set of post fields
	 * @return ModuleFields A ModuleFields object, containg the fields to render as well as any additional HTML markup to include
	 */
	public function getAdminAddFields($package, $vars=null) {
		return new ModuleFields();
	}
	
	/**
	 * Returns all fields to display to a client attempting to add a service with the module
	 *
	 * @param stdClass $package A stdClass object representing the selected package
	 * @param $vars stdClass A stdClass object representing a set of post fields
	 * @return ModuleFields A ModuleFields object, containg the fields to render as well as any additional HTML markup to include
	 */	
	public function getClientAddFields($package, $vars=null) {
		return new ModuleFields();		
	}
	
	/**
	 * Returns all fields to display to an admin attempting to edit a service with the module
	 *
	 * @param stdClass $package A stdClass object representing the selected package
	 * @param $vars stdClass A stdClass object representing a set of post fields
	 * @return ModuleFields A ModuleFields object, containg the fields to render as well as any additional HTML markup to include
	 */	
	public function getAdminEditFields($package, $vars=null) {
		Loader::loadHelpers($this, array("Html"));
		
		$fields = new ModuleFields();
		
		// Create license key label
		$license_key = $fields->label(Language::_("LicenseModule.service_field.license_module_key", true), "license_module_key");
		// Create license key field and attach to label
		$license_key->attach($fields->fieldText("license_module_key", $this->Html->ifSet($vars->license_module_key), array('id'=>"license_module_key")));
		// Set the label as a field
		$fields->setField($license_key);
		
		// Create license status label
		$label = $fields->label(Language::_("LicenseModule.service_field.statuses", true));
		$license_status = $fields->label(Language::_("LicenseModule.service_field.statuses.reissue", true), "license_module_status");
		// Create license status field and attach the label
		$status = $fields->fieldCheckbox("license_module_status", "reissue", $this->Html->ifSet($vars->license_module_status) == "reissue", array('id'=>"license_module_status"));
		$status->setLabel($license_status);
		$label->attach($status);
		// Set the label as a field
		$fields->setField($label);
		
		return $fields;
	}
	
	/**
	 * Fetches the HTML content to display when viewing the service info in the
	 * admin interface.
	 *
	 * @param stdClass $service A stdClass object representing the service
	 * @param stdClass $package A stdClass object representing the service's package
	 * @return string HTML content containing information to display when viewing the service info
	 */
	public function getAdminServiceInfo($service, $package) {
		$row = $this->getModuleRow();
		
		// Load the view into this object, so helpers can be automatically added to the view
		$this->view = new View("admin_service_info", "default");
		$this->view->base_uri = $this->base_uri;
		$this->view->setDefaultView("components" . DS . "modules" . DS . "license_module" . DS);
		
		// Load the helpers required for this view
		Loader::loadHelpers($this, array("Form", "Html"));

		$this->view->set("module_row", $row);
		$this->view->set("package", $package);
		$this->view->set("service", $service);
		$this->view->set("service_fields", $this->serviceFieldsToObject($service->fields));
		
		return $this->view->fetch();
	}
	
	/**
	 * Fetches the HTML content to display when viewing the service info in the
	 * client interface.
	 *
	 * @param stdClass $service A stdClass object representing the service
	 * @param stdClass $package A stdClass object representing the service's package
	 * @return string HTML content containing information to display when viewing the service info
	 */
	public function getClientServiceInfo($service, $package) {
		$row = $this->getModuleRow();
		
		// Load the view into this object, so helpers can be automatically added to the view
		$this->view = new View("client_service_info", "default");
		$this->view->base_uri = $this->base_uri;
		$this->view->setDefaultView("components" . DS . "modules" . DS . "license_module" . DS);
		
		// Load the helpers required for this view
		Loader::loadHelpers($this, array("Form", "Html"));

		$this->view->set("module_row", $row);
		$this->view->set("package", $package);
		$this->view->set("service", $service);
		$this->view->set("service_fields", $this->serviceFieldsToObject($service->fields));
		
		return $this->view->fetch();
	}
	
	/**
	 * Returns all tabs to display to a client when managing a service whose
	 * package uses this module
	 *
	 * @param stdClass $package A stdClass object representing the selected package
	 * @return array An array of tabs in the format of method => title. Example: array('methodName' => "Title", 'methodName2' => "Title2")
	 */
	public function getClientTabs($package) {
		return array('tabManageLicense' => Language::_("LicenseModule.getClientTabs.tabManageLicense", true));
	}
	
	/**
	 * Manage License tab
	 *
	 * @param stdClass $package A stdClass object representing the current package
	 * @param stdClass $service A stdClass object representing the current service
	 * @param array $get Any GET parameters
	 * @param array $post Any POST parameters
	 * @param array $files Any FILES parameters
	 * @return string The string representing the contents of this tab
	 */
	public function tabManageLicense($package, $service, array $get=null, array $post=null, array $files=null) {
		$this->view = new View("tab_manage_license", "default");
		// Load the helpers required for this view
		Loader::loadHelpers($this, array("Form", "Html"));
		
		$vars = $this->serviceFieldsToObject($service->fields);
		
		// Save configurable options
		if (!empty($post)) {
			$vars = (object)$post;
			
			if (!isset($post['license_module_status']))
				$post['license_module_status'] = "";
			
			$this->updateLicenseData($service, array('status' => $post['license_module_status']));
		}
		
		$this->view->set("vars", $vars);
		
		$this->view->setDefaultView("components" . DS . "modules" . DS . "license_module" . DS);
		return $this->view->fetch();
	}
	
	/**
	 * Fetches the service containing the given license key
	 *
	 * @param int $module_id The ID of the module to fetch on
	 * @param string $license_key The license key to search services for
	 * @return mixed A stdClass object representing the service, false otherwise
	 */
	public function getLicense($module_id, $license_key) {
		Loader::loadModels($this, array("Services"));
		$services = $this->Services->searchServiceFields($module_id, "license_module_key", $license_key);
		
		if (empty($services))
			return false;
		return $services[0];
	}
	
	/**
	 * Returns license data for the given service
	 *
	 * @param stdClass A stdClass object representing the service
	 * @return array An array of license data including:
	 * 	- domain An array of domains set for the license
	 * 	- ip An array of IP addresses set for the license
	 * 	- path An array of install paths for the license
	 * 	- status The service status (active, canceled, pending, suspended)
	 * 	- version The version recorded for this service
	 * 	- allow_reissue "true" if the license can be reissued, false otherwise
	 * 	- label The label used to ID this license package
	 * 	- custom 
	 * 	- addons
	 */
	public function getLicenseData(stdClass $service) {
		$fields = $this->serviceFieldsToObject($service->fields);
		$row = $this->getModuleRow();
		
		// Determine the license type
		Configure::load("license_manager", PLUGINDIR . "license_manager" . DS . "config" . DS);
		$custom = array(
			'license_type' => (isset($service->package_pricing->period) ? $service->package_pricing->period : ""),
			'cancellation_date' => $service->date_canceled
		);
		
		// Determine if this client matches a trial license
		if (Configure::get("LicenseManager.trial_client_id") == $service->client_id)
			$custom['license_type'] = "trial";
		
		$addons = null;
		
		if (!isset($this->Services))
			Loader::loadModels($this, array("Services"));
		
		if (!isset($this->ModuleManager))
			Loader::loadModels($this, array("ModuleManager"));
		
		foreach ($this->Services->getAllChildren($service->id, "active") as $addon) {
			if (!$addons)
				$addons = array();
			
			$addon_fields = array();
			foreach ($addon->fields as $addon_field) {
				if ($addon_field->encrypted == "0")
					$addon_fields[$addon_field->key] = $addon_field->value;
			}
			
			$module = $this->ModuleManager->get($addon->package->module_id, false, false);
			
			$addons[] = array(
				'qty' => isset($addon->qty) ? $addon->qty : null,
				'fields' => $addon_fields,
				'module' => isset($module->class) ? $module->class : null,
				'package_id' => $addon->package->id
			);
		}
		
		return array(
			'domain' => isset($fields->license_module_domains) ? $fields->license_module_domains : null,
			'ip' => isset($fields->license_module_ips) ? $fields->license_module_ips : null,
			'path' => isset($fields->license_module_paths) ? $fields->license_module_paths : null,
			'status' => isset($service->status) ? $service->status : null,
			'version' => isset($fields->license_module_version) ? $fields->license_module_version : null,
			'allow_reissue' => isset($row->meta->allow_reissue) ? $row->meta->allow_reissue : "false",
			'label' => isset($row->meta->label) ? $row->meta->label : null,
			'custom' => $custom,
			'addons' => $addons
		);
	}
	
	/**
	 * Updates the license data for the service using the given fields
	 *
	 * @param stdClass A stdClass object representing the service
	 * @param array $vars An array of key/value pairs where each key is the field and each value is the value to save for that field
	 */
	public function updateLicenseData(stdClass $service, $vars) {
		$license = $this->getLicenseData($service);
		$row = $this->getModuleRow();
		
		Loader::loadModels($this, array("Services"));
		$fields = array(
			'callhome' => "license_module_callhome",
			'version' => "license_module_version",
			'status' => "license_module_status",
			'domain' => "license_module_domains",
			'ip' => "license_module_ips",
			'path' => "license_module_paths"
		);
		
		$instances = $row->meta->instances;
		
		// Increment installed instances provided by instance package
		if (isset($row->meta->instance_package)) {
			foreach ((array)$license['addons'] as $addon) {
				if ($addon['package_id'] == $row->meta->instance_package) {
					$instances += $addon['qty'];
					// Continue because there may be more services of this package
				}
			}
		}
		
		#
		# TODO: Look for support and updates package from $row->meta->support_package in $license['addons']
		#	If support and updates is active allow $vars['version'] to be updated past $license['version']
		#	Else only allow $vars['version'] to be updated past $license['version'] iff this is a patch release (x.x.N)
		#
		
		foreach ($fields as $key => $name) {
			if (array_key_exists($key, $vars)) {
				
				if ($key == "domain" || $key == "ip" || $key == "path") {
					if (!is_array($license[$key]))
						$license[$key] = array();
						
					if (count($license[$key]) < $instances && !in_array($vars[$key], $license[$key]))
						$license[$key][] = $vars[$key];
				}
				else
					$license[$key] = $vars[$key];
				
				$this->Services->editField($service->id, array('key' => $name, 'value' => $license[$key]));
			}
		}
	}

	/**
	 * Returns all license methods supported
	 *
	 * @return array An array of key/value pairs
	 */	
	private function licenseMethods() {
		return array(
			'default' => Language::_("LicenseModule.license_methods.default", true)
		);
	}
	
	/**
	 * Returns all license key methods supported
	 *
	 * @return array An array of key/value pairs
	 */
	private function licenseKeyMethods() {
		return array(
			'auto' => Language::_("LicenseModule.key_methods.auto", true)
		);
	}
}
?>